@@ -1,5 +1,3 @@
-/* $Xorg: process.c,v 1.6 2001/02/09 02:05:38 xorgcvs Exp $ */
-/* $XdotOrg: xc/programs/xauth/process.c,v 1.3 2004/04/24 23:26:55 alanc Exp $ */
 /*
 
 Copyright 1989, 1998  The Open Group
@@ -27,7 +25,6 @@ other dealings in this Software without prior written authorization
 from The Open Group.
 
 */
-/* $XFree86: xc/programs/xauth/process.c,v 3.23 2003/11/25 03:15:04 dawes Exp $ */
 
 /*
  * Author:  Jim Fulton, MIT X Consortium
@@ -485,9 +482,6 @@ get_displayname_auth(char *displayname, AuthList **authl)
     if (cp && strncmp (cp, "/unix:", 6) == 0)
       prelen = (cp - displayname);
     
-    if (strncmp (displayname, "/tmp/launch", 11) == 0)
-        displayname = strrchr(displayname, '/') + 1;
-
     if (!parse_displayname (displayname + ((prelen > 0) ? prelen + 1 : 0),
 			    &family, &host, &dpynum, &scrnum, &rest)) {
 	return False;
@@ -564,8 +558,8 @@ cvthexkey(char *hexstr, char **ptrp)	/* turn hex key string into octets */
 	len++;
     }
 
-    /* if odd then there was an error */
-    if ((len & 1) == 1) return -1;
+    /* if 0 or odd, then there was an error */
+    if (len == 0 || (len & 1) == 1) return -1;
 
 
     /* now we know that the input is good */
@@ -769,7 +763,7 @@ auth_initialize(char *authfilename)
 	    return -1;
 	}				/* else ignore it */
 	fprintf (stderr, 
-		 "%s:  creating new authority file %s\n",
+		 "%s:  file %s does not exist\n",
 		 ProgramName, authfilename);
     } else {
 	xauth_existed = True;
@@ -894,7 +888,8 @@ auth_finalize(void)
 #if defined(WIN32) || defined(__UNIXOS2__)
 		if (rename(temp_name, xauth_filename) == -1)
 #else
-		if (link (temp_name, xauth_filename) == -1)
+		/* Attempt to rename() if link() fails, since this may be on a FS that does not support hard links */
+		if (link (temp_name, xauth_filename) == -1 && rename(temp_name, xauth_filename) == -1)
 #endif
 		{
 		    fprintf (stderr,
@@ -960,7 +955,7 @@ fprintfhex(register FILE *fp, int len, char *cp)
     char *hex;
 
     hex = bintohex(len, cp);
-    fprintf(fp, hex);
+    fprintf(fp, "%s", hex);
     free(hex);
 }
 
@@ -1500,6 +1495,7 @@ do_extract(char *inputfilename, int lineno, int argc, char **argv)
 
     ed.fp = NULL;
     ed.filename = argv[1];
+    ed.used_stdout = False;
     ed.numeric = (argv[0][0] == 'n');
     ed.nwritten = 0;
     ed.cmd = argv[0];
